# coding:utf-8

import os
from typing import Optional
from urllib.parse import urljoin

from requests.exceptions import HTTPError
from sitepages import page


class rfc_page(page):
    BASE: str = "https://www.rfc-editor.org"

    def __init__(self, location: str, filepath: Optional[str] = None):
        super().__init__(urljoin(self.BASE, location))
        self.__filepath: str = filepath or location

    @property
    def filepath(self) -> str:
        return self.__filepath

    def save(self) -> bool:
        rewrite: bool = not os.path.isfile(self.filepath)
        dirname: str = os.path.dirname(self.filepath)
        if not os.path.exists(dirname):
            os.makedirs(dirname)

        try:
            datas: bytes = super().fetch().content
        except HTTPError:
            return False

        if not rewrite:
            with open(self.filepath, "rb") as rhdl:
                if rhdl.read() != datas:
                    rewrite = True

        if rewrite:
            with open(self.filepath, "wb") as whdl:
                whdl.write(datas)

        return os.path.isfile(self.filepath)


class rfc_text(rfc_page):
    def __init__(self, number: int):
        self.__file: str = f"rfc{number}.txt"
        super().__init__(os.path.join("rfc", self.file))

    @property
    def file(self) -> str:
        return self.__file


class rfc_html(rfc_page):
    def __init__(self, number: int):
        self.__file: str = f"rfc{number}.html"
        self.__link: str = os.path.join("rfc", f"rfc{number}")
        super().__init__(os.path.join("rfc", self.file))

    @property
    def file(self) -> str:
        return self.__file

    @property
    def link(self) -> str:
        return self.__link

    def save(self) -> bool:
        if not super().save():
            return False

        if not os.path.exists(self.link) or os.readlink(self.link) != self.file:  # noqa:E501
            os.symlink(self.file, self.link)

        return os.readlink(self.link) == self.file


class rfc_pdf(rfc_page):
    def __init__(self, number: int):
        self.__file: str = os.path.join("pdfrfc", f"rfc{number}.txt.pdf")
        super().__init__(self.file, os.path.join("rfc", self.file))

    @property
    def file(self) -> str:
        return self.__file


class rfc():
    def __init__(self, number: int):
        self.__number: int = number

    @property
    def number(self) -> int:
        return self.__number

    @property
    def text(self) -> rfc_text:
        return rfc_text(self.number)

    @property
    def html(self) -> rfc_html:
        return rfc_html(self.number)

    @property
    def pdfrfc(self) -> rfc_pdf:
        return rfc_pdf(self.number)



# coding:utf-8

from concurrent.futures import ThreadPoolExecutor
from typing import List
from typing import Optional
from typing import Sequence

from xarg import add_command
from xarg import argp
from xarg import commands
from xarg import run_command

from .attribute import __description__
from .attribute import __urlhome__
from .attribute import __version__
from .rfc_editor import rfc


@add_command("rfc-download", help="Downlaod RFCs")
def add_cmd_download(_arg: argp):
    _arg.add_argument("--text", action="store_true", help="Download text RFC")
    _arg.add_argument("--html", action="store_true", help="Download html RFC")
    _arg.add_argument("--pdf", action="store_true", help="Download pdf RFC")
    _arg.add_pos("rfc_number", type=int, nargs="+", metavar="RFC", help="number")  # noqa:E501


@run_command(add_cmd_download)
def run_cmd_download(cmds: commands) -> int:
    rfc_nums: List[int] = cmds.args.rfc_number
    all_rfcs: bool = not (cmds.args.text or cmds.args.text or cmds.args.text)
    with ThreadPoolExecutor(max_workers=64) as executor:
        for number in rfc_nums:
            site: rfc = rfc(number)
            if all_rfcs or cmds.args.text:
                executor.submit(site.text.save)
            if all_rfcs or cmds.args.html:
                executor.submit(site.html.save)
            if all_rfcs or cmds.args.pdf:
                executor.submit(site.pdfrfc.save)
    return 0


def download(argv: Optional[Sequence[str]] = None) -> int:
    cmds = commands()
    cmds.version = __version__
    return cmds.run(
        root=add_cmd_download,
        argv=argv,
        description=__description__,
        epilog=f"For more, please visit {__urlhome__}.")



# coding:utf-8

from urllib.parse import urljoin

__project__ = "rfc-kits"
__version__ = "0.1.2"
__description__ = "Search, View and Download RFCs."
__urlhome__ = "https://github.com/bondbox/rfc-kits"
__urlcode__ = __urlhome__
__urldocs__ = __urlhome__
__urlbugs__ = urljoin(__urlhome__, "issues")

# author
__author__ = "Mingzhe Zou"
__author_email__ = "zoumingzhe@outlook.com"
